; TIP1043.LSP:  NUKE.LSP    Explode All Blocks   (c)1994, M. Jenkins

;; This is a command for exploding all blocks in a       
;; drawing. It will get all blocks regardless of how deep
;; they are nested. It automatically filters out hatch   
;; patterns and dimensions (anonymous blocks).           
;;

(defun C:NUKE (/ _clean _star sset index sset ent wh _espace _cspace)

   ;;
   ;; Purpose: Determines if the entity is in model or paper space.
   ;;  Passed: Nothing.
   ;; Returns: 1 if it's in paper space, 0 if it's in model space.                                            
   ;;
   (defun _espace (ent)
      (if (not (assoc 67 (entget ent)))
         0
         (cdr (assoc 67 (entget ent)))
      )
   )  

   ;;
   ;; Purpose: Determines the current drawing space.
   ;;  Passed: Nothing.
   ;; Returns: 1 if in paper space, 0 if in model space.                                            
   ;;
   (defun _cspace ()
      (if (= (getvar "tilemode") 1)
         0
         (if (= (getvar "cvport") 1)
            1
            0
         )
      )  
   ) 

   ;;
   ;; Purpose: Provides a blinking star. 
   ;;  Passed: Nothing.
   ;; Returns: Nothing of value.                                            
   ;;
   (defun _star ()
      (prompt (strcat "\rExploding all blocks..." wh))
      (if 
         (= wh "*")
         (setq wh " ")
         (setq wh "*")
      )
   )      

   ;;
   ;; Purpose: Cleans the selections set. 
   ;;  Passed: A selection set.
   ;; Returns: The new selection set if successful, nil if not.                                            
   ;;
   (defun _clean (sset / sset ent x y z name ssnew)
      (if sset
         (progn
            (setq 
               index -1
               ssnew (ssadd)
            )
            (while 
               (setq ent (ssname sset (setq index (1+ index))))
               (setq 
                  x (cdr (assoc 41 (entget ent)))
                  y (cdr (assoc 42 (entget ent)))
                  z (cdr (assoc 43 (entget ent)))
                  name (cdr (assoc 2 (entget ent)))
               )
               (if 
                  (and 
                     (= x y z) 
                     (/= (substr name 1 1) "*")
                     (= (_espace ent) (_cspace))
                  )   
                  (setq ssnew (ssadd ent ssnew))
               )
               (_star)
            )
            (if 
               (/= (sslength ssnew) 0)
               ssnew
               nil
            )
         )
         nil
      )
   )

   (setq 
      func_name "NUKE"
      wh "*"
   )
   (prompt "\nExploding all blocks...")
   (while 
      (setq
         sset
         (_clean
            (ssget "X"
               '(
                  (-4 . "<AND")
                  (0 . "INSERT")
                  (-4 . ">")
                  (41 . 0)
                  (-4 . ">")
                  (42 . 0)
                  (-4 . ">")
                  (43 . 0)
                  (-4 . "AND>")  
               )
            )   
         )   
      )
      (setq index -1)
      (while 
         (setq ent (ssname sset (setq index (1+ index))))
         (command "_.explode" ent)
         (_star)
      )
   )
   (prompt "\rExploding all blocks...DONE!")
   (prin1)
)
(princ "NUKE loaded.")
(princ)
